/*
 * pthread_sigmask.c
 *
 * Copyright (C) 2005 Dan Williams and Red Hat, Inc., All rights reserved
 *
 * Thin wrapper around pthread_sigmask to counter signal blocking in Python <= 2.3
 */
#include <Python.h>
#include <signal.h>

static char module_doc[] = "\nThin pthread_sigmask wrapper.\n";

static PyObject *
pthread_sigmask_pthread_sigmask (PyObject *self, PyObject *args)
{
    int how;
    PyObject *set_list = NULL;
    long set_list_len;
    PyObject *oset_list;
    int i, err;
    sigset_t set, oset;
    sigset_t *set_ptr = NULL;

    if (!PyArg_ParseTuple (args, "i|O!:pthread_sigmask", &how, &PyList_Type, &set_list))
        return NULL;

    sigemptyset (&set);
    sigemptyset (&oset);

    if (set_list)
    {
        set_list_len = PyList_Size (set_list);

        for (i = 0; i < set_list_len; i++)
        {
            PyObject *py_sig;
            
            py_sig = PyList_GetItem (set_list, i);
            if (!PyInt_Check (py_sig))
            {
                PyErr_BadArgument ();
                return NULL;
            }
            sigaddset (&set, PyInt_AsLong (py_sig));
        }
        set_ptr = &set;
    }

    oset_list = PyList_New (0);
    err = pthread_sigmask (how, set_ptr, &oset);
    if (err == 0)
    {
        /* Convert oset to a python list */
        for (i = 1; i < _NSIG; i++)
        {
            if (sigismember (&oset, i) == 1)
                PyList_Append (oset_list, PyInt_FromLong (i));
        }
    }

    return Py_BuildValue ("(iO)", err, oset_list);
}
PyDoc_STRVAR (pthread_sigmask_doc,
"Set the current thread's signal mask.  See pthread_sigmask(P) manpage.\n\
\n\
Arguments: how - One of SIG_BLOCK, SIG_SETMASK, SIG_UNBLOCK\n\
           set (optional) - Python list of signals to block, ex [signal.SIGALRM]\n\
Returns:   (result, output set), where result = 0 on success, and\n\
              output set is the previous signal mask as a list\n\
");


/* Methods in the pthread_sigmask module */
static PyMethodDef pthread_sigmask_methods[] = {
    { "pthread_sigmask", (PyCFunction) pthread_sigmask_pthread_sigmask, METH_VARARGS, pthread_sigmask_doc },
    { NULL, NULL }
};


/*
 * Initialize the sigmask module
 *
 * Arguments: None
 * Returns:   None
 */
PyMODINIT_FUNC
initpthread_sigmask (void)
{
    PyObject *m, *d, *x;

    m = Py_InitModule3 ("pthread_sigmask", pthread_sigmask_methods, module_doc);

    /* Add some symbolic constants to the module */
    d = PyModule_GetDict (m);

    x = PyInt_FromLong (SIG_BLOCK);
    PyDict_SetItemString (d, "SIG_BLOCK", x);
    Py_XDECREF (x);

    x = PyInt_FromLong (SIG_SETMASK);
    PyDict_SetItemString (d, "SIG_SETMASK", x);
    Py_XDECREF (x);

    x = PyInt_FromLong (SIG_UNBLOCK);
    PyDict_SetItemString (d, "SIG_UNBLOCK", x);
    Py_XDECREF (x);
}

